@@ -134,6 +134,13 @@ class ThreadedServer(multiprocessing.Process):
         ]
         return all(conditions)
 
+    def should_serve_admin_media(self):
+        conditions = [
+            'django.contrib.admin' in settings.INSTALLED_APPS,
+            getattr(settings, 'LETTUCE_SERVE_ADMIN_MEDIA', False),
+        ]
+        return any(conditions)
+
     def run(self):
         self.lock.acquire()
         pidfile = os.path.join(tempfile.gettempdir(), 'lettuce-django.pid')
@@ -178,7 +185,7 @@ class ThreadedServer(multiprocessing.Process):
             )
 
         handler = WSGIHandler()
-        if 'django.contrib.admin' in settings.INSTALLED_APPS:
+        if self.should_serve_admin_media():
             admin_media_path = ''
             handler = AdminMediaHandler(handler, admin_media_path)
 
@@ -211,8 +218,12 @@ class Server(object):
     def start(self):
         """Starts the webserver thread, and waits it to be available"""
         call_hook('before', 'runserver', self._actual_server)
-        if 'django.contrib.admin' in settings.INSTALLED_APPS:
-            print "Preparing to serve django's admin site static files..."
+        if self._actual_server.should_serve_admin_media():
+            msg = "Preparing to serve django's admin site static files"
+            if getattr(settings, 'LETTUCE_SERVE_ADMIN_MEDIA', False):
+                msg += ' (as per settings.LETTUCE_SERVE_ADMIN_MEDIA=True)'
+
+            print "%s..." % msg
 
         self._actual_server.start()
         self._actual_server.wait()
